import re

def exists(filename):
	"""check whether the file is exists or not"""
	import os
	if os.path.exists(filename): return True
	else: return False

def create(filename):
	"""creates new file"""
	import os
	if not os.path.exists(filename):
		with open(filename,'x') as f:
			pass
		return True
	else: return False

def flush(filename,stream=False):
	"""flush file"""
	if not stream:
		import os
		if os.path.exists(filename):
			os.remove(filename)
			with open(filename,'x') as f:
				pass
			return True
		else: return False
	elif stream:
		with open(filename,'w') as f:
			f.write('')
		return True

def remove(filename):
	"""remove/deletes file"""
	import os
	if os.path.exists(filename):
		os.remove(filename)
		return True
	else: return False

def dump(filename,set):
	"""dump a dictionary or a set to INI file format"""
	with open(filename,'w+') as f:
		for ns in set:
			if isinstance(set[ns], dict):
				f.write(f'[{ns}]\n')
				for ps in set[ns]:
					if isinstance(set[ns][ps], dict) or isinstance(set[ns][ps], list): continue
					f.write(f'{ps}={set[ns][ps]}\n')
			else:
				f.write(f"{ns}={set[ns]}\n")

def dump_bin(filename,set):
	"""dump a dictionary or a set to INI file format"""
	with open(filename,'w+') as f:
		f.write('INI\n') # ini format binary
		for ns in set:
			if isinstance(set[ns], dict):
				f.write(f'[{ns}]\n')
				for ps in set[ns]:
					if isinstance(set[ns][ps], dict) or isinstance(set[ns][ps], list): continue
					f.write(f'{ps}={set[ns][ps]}\n')
			else:
				f.write(f"{ns}={set[ns]}\n")

def parse_property(string):
	if check_comment(string): return
	prop = re.split(r'\s*(\=)\s*',string)
	if len(prop) < 3: return
	del prop[1]
	key, val = prop[0], prop[1]
	_key = re.split(r'(^[#])|((.*)\s[#]$)',key)[0]
	val = ''.join(prop[1:])
	val = re.split(r'((.)^[#]$)|\s([#])',val)[0]
	if key == _key:
		return key, val

def parse_section(string):
	if check_comment(string): return
	sec = re.split(r'^\s*\[(.*)\]|[#;]$',string)
	if sec[0] != string:
		for i,s in enumerate(sec):
			if not s: del sec[i]
		_sec = re.split(r'((.*)\s[#])',sec[0])[0]
		if _sec: return _sec

def check_comment(string):
	sec = re.match(r'^[#;]',string)
	if sec: return True
	return False

def is_property(string):
	if parse_property(string) != None: return True
	return False

def is_section(string):
	if parse_section(string) != None: return True
	return False

def is_ini(filename):
	return filename.endswith('.ini')


import setuptools

def read(fname):
	with open(fname,'r') as f:
		return f.read()

setuptools.setup(
name='iniparser2',
version='1.9.4',
author='HugeBrain16',
author_email='joshtuck373@gmail.com',
description='An INI parser or config parser',
license='MIT',
keywords='iniparser configparser ini config parser file',
url='https://github.com/HugeBrain16/iniparser2',
project_url= {
		"Bug Tracker": 'https://github.com/HugeBrain16/iniparser2/issues'	
	},
packages=setuptools.find_packages(),
long_description=read('README.md'),
long_description_content_type='text/markdown',
classifiers=[
		'Development Status :: 5 - Production/Stable',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python :: 3',
	]
)


# uhh... ¯\_(ツ)_/¯

class ParsingError(Exception):
	pass

class INI:
	def __init__(self, filename):
		self.filename = str(filename)

	def __enter__(self):
		return INI(self.filename)

	def __exit__(*args,**kwargs): #hmm...
		pass

	def read(self):
		"""read sections and properties"""
		return parse(open(self.filename,'r').read())

	def write(self,sets):
		"""write properties and sections to file"""
		from .utils import dump
		if not isinstance(sets,dict): raise TypeError("INI properties must be a dict object")
		dump(self.filename,sets); return True

class INI_BIN:
	def __init__(self, filename):
		self.filename = str(filename)

	def __enter__(self):
		return INI_BIN(self.filename)

	def __exit__(*args,**kwargs): #hmm...
		pass

	def read(self):
		"""read sections and properties in binary format"""
		import marshal, io
		data = marshal.load(open(self.filename,'rb')).decode('utf-8')
		_data = io.StringIO(data).readlines()
		if _data[0].strip() != "INI": return TypeError("Binary file is not an INI format")
		return parse(data)

	def write(self,sets):
		"""write properties and sections to file in binary format"""
		from .utils import dump_bin; import marshal
		if not isinstance(sets,dict): raise TypeError("INI properties must be a dict object")
		dump_bin(self.filename,sets)
		raw_data = open(self.filename,'r').read().encode('utf-8')
		marshal.dump(raw_data,open(self.filename,'wb'))

def parse(string):
	"""beans for everyone, haha... :|"""
	from .utils import parse_section,parse_property,is_section,is_property,check_comment
	import io
	
	ret = dict()
	lines,point,anchor,fsec=io.StringIO(string).readlines(),0,0,False

	for idx, line, in enumerate(lines):
		if line.strip() == 'INI' or not line.strip(): continue # skip INI file format for binary, i guess...

		if is_section(line.strip()) or fsec:
			fsec=True
			_section = parse_section(line.strip())
			point,anchor=idx+1,idx+1

			for i in range(anchor,len(lines)):
				anchor += 1
				if is_section(lines[i].strip()):
					break
			
			if _section: ret.update({_section: {}})
			
			for i in range(point,anchor):
				if not lines[i].strip(): continue

				if is_property(lines[i].strip()):
					key, val = parse_property(lines[i].strip())

					if not key:
						raise ParsingError("invalid property key name at line {lineno}".format(lineno=i+1))

					if _section != None: ret[_section].update({key:val})
				else:
					if is_section(lines[i].strip()): continue

					if lines[i].strip() and not check_comment(lines[i].strip()): raise ParsingError("error parsing property at line {lineno}".format(lineno=i+1))

		if not fsec:
			if is_property(line.strip()):
				key, val = parse_property(line.strip())

				if not key:
					raise ParsingError("invalid property key name at line {lineno}".format(lineno=idx+1))

				ret.update({key: val})
			else:
				if not check_comment(line.strip()): raise ParsingError("error parsing property at line {lineno}".format(lineno=idx+1))
	
	return ret